@@ -442,18 +442,13 @@ struct Grid : GridBase<T> {
 
   void set_value(int u, int v, int w, T x) { data[index_s(u, v, w)] = x; }
 
-  template <bool UsePbc, typename Func>
-  void use_points_in_box(Fractional fctr, int du, int dv, int dw,
-                         Func&& func, bool fail_on_too_large_radius=true) {
+  template <bool UsePbc>
+  void check_size_for_points_in_box(int& du, int& dv, int& dw,
+                                    bool fail_on_too_large_radius) const {
     if (fail_on_too_large_radius) {
       if (2 * du >= nu || 2 * dv >= nv || 2 * dw >= nw)
         fail("grid operation failed: radius bigger than half the unit cell?");
     }
-    if (UsePbc)
-      fctr = fctr.wrap_to_unit();
-    int u0 = iround(fctr.x * nu);
-    int v0 = iround(fctr.y * nv);
-    int w0 = iround(fctr.z * nw);
     if (UsePbc && !fail_on_too_large_radius) {
       // If we'd use the minimum image convention the max would be (nu-1)/2.
       // The limits set here are necessary for index_n() that is used below.
@@ -461,6 +456,15 @@ struct Grid : GridBase<T> {
       dv = std::min(dv, nv - 1);
       dw = std::min(dw, nw - 1);
     }
+  }
+
+  template <bool UsePbc, typename Func>
+  void do_use_points_in_box(Fractional fctr, int du, int dv, int dw, Func&& func) {
+    if (UsePbc)
+      fctr = fctr.wrap_to_unit();
+    int u0 = iround(fctr.x * nu);
+    int v0 = iround(fctr.y * nv);
+    int w0 = iround(fctr.z * nw);
     int u_lo = u0 - du;
     int u_hi = u0 + du;
     int v_lo = v0 - dv;
@@ -485,6 +489,13 @@ struct Grid : GridBase<T> {
         }
   }
 
+  template <bool UsePbc, typename Func>
+  void use_points_in_box(Fractional fctr, int du, int dv, int dw,
+                         Func&& func, bool fail_on_too_large_radius=true) {
+    check_size_for_points_in_box<UsePbc>(du, dv, dw, fail_on_too_large_radius);
+    do_use_points_in_box<UsePbc>(fctr, du, dv, dw, func);
+  }
+
   template <bool UsePbc, typename Func>
   void use_points_around(const Fractional& fctr_, double radius, Func&& func,
                          bool fail_on_too_large_radius=true) {
@@ -510,7 +521,7 @@ struct Grid : GridBase<T> {
 
   void change_values(T old_value, T new_value) {
     for (auto& d : data)
-      if (d == old_value)
+      if (impl::is_same(d, old_value))
         d = new_value;
   }
 
@@ -613,18 +624,13 @@ struct Grid : GridBase<T> {
     return mask;
   }
 
-  MaskedGrid<T> asu();
+  MaskedGrid<T> masked_asu();
 };
 
 
 template<typename T, typename V> struct MaskedGrid {
+  std::vector<V> mask;
   Grid<T>* grid;
-  Grid<V> mask; // should we simply store the mask as vector?
-
-  MaskedGrid(Grid<T>& grid_, std::vector<V>&& mask_data) : grid(&grid_) {
-    mask.copy_metadata_from(grid_);
-    mask.data = mask_data;
-  }
 
   struct iterator {
     MaskedGrid& parent;
@@ -642,8 +648,7 @@ template<typename T, typename V> struct MaskedGrid {
             ++w;
           }
         }
-      } while (index != parent.mask.data.size() &&
-               parent.mask.data[index] != 0);
+      } while (index != parent.mask.size() && parent.mask[index] != 0);
       return *this;
     }
     typename GridBase<T>::Point operator*() {
@@ -653,12 +658,23 @@ template<typename T, typename V> struct MaskedGrid {
     bool operator!=(const iterator &o) const { return index != o.index; }
   };
   iterator begin() { return {*this, 0}; }
-  iterator end() { return {*this, mask.data.size()}; }
+  iterator end() { return {*this, mask.size()}; }
 };
 
+template<typename T> MaskedGrid<T> Grid<T>::masked_asu() {
+  return {get_asu_mask<std::int8_t>(), this};
+}
+
+
 template<typename T>
-MaskedGrid<T> Grid<T>::asu() {
-  return {*this, get_asu_mask<std::int8_t>()};
+Correlation calculate_correlation(const GridBase<T>& a, const GridBase<T>& b) {
+  if (a.data.size() != b.data.size() || a.nu != b.nu || a.nv != b.nv || a.nw != b.nw)
+    fail("calculate_correlation(): grids have different sizes");
+  Correlation corr;
+  for (size_t i = 0; i != a.data.size(); ++i)
+    if (!std::isnan(a.data[i]) && !std::isnan(b.data[i]))
+      corr.add_point(a.data[i], b.data[i]);
+  return corr;
 }
 
 } // namespace gemmi
